use rustracing_jaeger::Span;
use tiny_http::{Request, ResponseBox};

use crate::{
    db::{DBConnection, UserRole},
    dto,
    routes::{filters::UserInfo, get_reply, AppError}
};

pub fn users(_: &Span, _: &mut Request, db: &mut DBConnection, info: UserInfo) -> Result<ResponseBox, AppError> {
    if info.0.role != UserRole::Admin {
        return AppError::Forbidden("Forbidden").err();
    }

    let users = db.get_users();

    let mut res = dto::responses::AdminUsers {
        statusCode: 200,
        users: Vec::new()
    };

    for user in users {
        res.users.push(dto::responses::AdminUsersEntry {
            id: user.id,
            gitlab: user.gitlab,
            name: user.name,
            role: user.role,
            tfaEnabled: user.tfa_type != crate::db::TfaTypes::None
        });
    }

    get_reply(&res)
}

pub fn set_role(
    span: &Span,
    _: &mut Request,
    db: &mut DBConnection,
    info: UserInfo,
    data: dto::requests::AdminSetRole
) -> Result<ResponseBox, AppError> {
    if info.0.role != UserRole::Admin {
        return AppError::Forbidden("Forbidden").err();
    }

    let mut user = db.get_user(span, data.user).ok_or(AppError::Forbidden("Invalid user"))?;
    user.role = data.role;
    db.save_user(span, &user);

    get_reply(&dto::responses::Success { statusCode: 200 })
}

pub fn logout(
    span: &Span,
    _: &mut Request,
    db: &mut DBConnection,
    info: UserInfo,
    data: dto::requests::Admin
) -> Result<ResponseBox, AppError> {
    if info.0.role != UserRole::Admin {
        return AppError::Forbidden("Forbidden").err();
    }

    db.delete_all_tokens(span, data.user);

    get_reply(&dto::responses::Success { statusCode: 200 })
}

pub fn delete_user(
    span: &Span,
    _: &mut Request,
    db: &mut DBConnection,
    info: UserInfo,
    data: dto::requests::Admin
) -> Result<ResponseBox, AppError> {
    if info.0.role != UserRole::Admin {
        return AppError::Forbidden("Forbidden").err();
    }

    let user = db.get_user(span, data.user).ok_or(AppError::Forbidden("Invalid user"))?;

    db.delete_all_tokens(span, data.user);

    let root_node =
        super::fs::get_node_and_validate(span, &user, user.root_id, db).expect("Failed to get root node for deleting");

    super::fs::delete_node_root(span, &root_node, db);

    db.delete_user(&user);

    get_reply(&dto::responses::Success { statusCode: 200 })
}

pub fn disable_2fa(
    span: &Span,
    _: &mut Request,
    db: &mut DBConnection,
    info: UserInfo,
    data: dto::requests::Admin
) -> Result<ResponseBox, AppError> {
    if info.0.role != UserRole::Admin {
        return AppError::Forbidden("Forbidden").err();
    }

    let mut user = db.get_user(span, data.user).ok_or(AppError::Forbidden("Invalid user"))?;

    user.tfa_type = crate::db::TfaTypes::None;
    db.save_user(span, &user);

    get_reply(&dto::responses::Success { statusCode: 200 })
}

pub fn is_admin(_: &Span, _: &mut Request, _: &mut DBConnection, info: UserInfo) -> Result<ResponseBox, AppError> {
    if info.0.role != UserRole::Admin {
        return AppError::Forbidden("Forbidden").err();
    }

    get_reply(&dto::responses::Success { statusCode: 200 })
}

pub fn get_token(
    span: &Span,
    _: &mut Request,
    db: &mut DBConnection,
    info: UserInfo,
    user: i32
) -> Result<ResponseBox, AppError> {
    if info.0.role != UserRole::Admin {
        return AppError::Forbidden("Forbidden").err();
    }

    let user = db.get_user(span, user).ok_or(AppError::Forbidden("Invalid user"))?;

    get_reply(&dto::responses::Login {
        statusCode: 200,
        jwt: super::auth::get_token(span, &user, db)
    })
}
