import type { Requests, Responses, UploadFile } from '@/dto';
import { get_token, post_token, isErrorResponse } from './base';
import axios from 'axios';

export const get_root = (
	token: string
): Promise<Responses.GetRoot | Responses.Error> =>
	get_token('/api/fs/root', token);

export const get_node = (
	token: string,
	node: number
): Promise<Responses.GetNode | Responses.Error> =>
	get_token(`/api/fs/node/${node}`, token);

export const get_path = (
	token: string,
	node: number
): Promise<Responses.GetPath | Responses.Error> =>
	get_token(`/api/fs/path/${node}`, token);

export const create_folder = (
	token: string,
	parent: number,
	name: string
): Promise<
	Responses.CreateFolder | Responses.CreateFolderExists | Responses.Error
> =>
	post_token<Requests.CreateFolder>(
		'/api/fs/create_folder',
		{
			parent: parent,
			name: name
		},
		token
	);

export const create_file = (
	token: string,
	parent: number,
	name: string
): Promise<
	Responses.CreateFolder | Responses.CreateFolderExists | Responses.Error
> =>
	post_token<Requests.CreateFolder>(
		'/api/fs/create_file',
		{
			parent: parent,
			name: name
		},
		token
	);

export const move = (
	token: string,
	nodes: number[],
	target: number
): Promise<Responses.Success | Responses.Error> =>
	post_token<Requests.Move>(
		'/api/fs/move',
		{
			nodes: nodes,
			target: target
		},
		token
	);

export const create_zip = (
	token: string,
	nodes: number[]
): Promise<Responses.CreateZip | Responses.Error> =>
	post_token<Requests.CreateZip>(
		'/api/fs/create_zip',
		{
			nodes: nodes
		},
		token
	);

export const download_preview = (
	token: string,
	node: number
): Promise<Responses.DownloadBase64 | Responses.Error> =>
	get_token(`/api/fs/download_preview/${node}`, token);

export const get_type = (
	token: string,
	node: number
): Promise<Responses.GetType | Responses.Error> =>
	get_token(`/api/fs/get_type/${node}`, token);

export async function upload_file(
	token: string,
	file: UploadFile,
	onProgress: (progressEvent: ProgressEvent) => void
): Promise<[Responses.Success | Responses.Error, boolean]> {
	const node = await create_file(token, file.parent, file.file.name);
	if (isErrorResponse(node)) return [node, false];
	if ('exists' in node && !node.isFile)
		return [{ statusCode: 400, message: 'File exists as folder' }, false];

	return [
		await axios
			.post(`/api/fs/upload/${node.id}`, file.file, {
				headers: {
					Authorization: 'Bearer ' + token,
					'Content-type': 'multipart/form-data'
				},
				onUploadProgress: onProgress
			})
			.then((res) => {
				console.log(res);
				return res.data;
			})
			.catch((err) => {
				console.log(err);
				return err.response.data;
			}),
		'exists' in node
	];
}

export function download_file(token: string, id: number) {
	const form = document.createElement('form');
	form.method = 'post';
	form.target = '_blank';
	form.action = '/api/fs/download';
	form.innerHTML = `<input type="hidden" name="jwtToken" value="${token}"><input type="hidden" name="id" value="${id}">`;
	document.body.appendChild(form);
	form.submit();
	document.body.removeChild(form);
}

export function download_multi_file(token: string, ids: number[]) {
	const form = document.createElement('form');
	form.method = 'post';
	form.target = '_blank';
	form.action = '/api/fs/download_multi';
	form.innerHTML = `<input type="hidden" name="jwtToken" value="${token}"><input type="hidden" name="id" value="${ids.join(
		','
	)}">`;
	document.body.appendChild(form);
	form.submit();
	document.body.removeChild(form);
}
